package org.bib.jeffrey.impl;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;

import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bib.jeffrey.TiffService;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.stereotype.Service;

import com.twelvemonkeys.io.FileUtil;

@Service
public class TiffServiceImpl implements TiffService {
// Constructors
	/**
	 * Default
	 */
	public TiffServiceImpl() {
	}
	
// Public Methods
	@Override
	public void toMultiPageTiff(String pathPattern, File outputFile, String compressionType) {
		List<BufferedImage> images = loadImages(pathPattern);
		
		byte[] tiffBytes = createMultiPageTiff(images, compressionType);
		
		writeBytesToFile(outputFile, tiffBytes);
	}
	
	
// Packages Methods
	@Override
	public List<BufferedImage> loadImages(String pathPattern) {
		// Load from classpath
		ClassLoader classLoader = this.getClass().getClassLoader();
		
		ResourcePatternResolver resolver = new PathMatchingResourcePatternResolver(classLoader);
		
		Resource[] imgResources = null;
		
		try {
			imgResources = resolver.getResources(pathPattern);
		}
		catch(Exception e) {
			throw new RuntimeException("Failed to laod images from classpath.", e);
		}
		
		// Convert buffered images
		List<BufferedImage> imageList = Arrays.asList(imgResources).stream()
				.map(r -> {
					BufferedImage bufferedImage = null;
					
					try(InputStream imgIn = r.getInputStream()) {
						bufferedImage = ImageIO.read(imgIn);
					}
					catch(Exception e) {
						throw new RuntimeException("Failed to read image.", e);
					}
					
					return bufferedImage;
				})
				.collect(Collectors.toList());
		
		return imageList;
	}

	@Override
	public byte[] createMultiPageTiff(List<BufferedImage> images, String compressionType) {
		ByteArrayOutputStream buffer = new ByteArrayOutputStream();
		ImageOutputStream output = null;
		
        try {
        	output = ImageIO.createImageOutputStream(buffer);
        	
        	ImageWriter writer = ImageIO.getImageWritersByFormatName("tiff").next();
            writer.setOutput(output);

            ImageWriteParam params = writer.getDefaultWriteParam();
            params.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
            params.setCompressionType(compressionType);
            
            // Combine the tiffs
            writer.prepareWriteSequence(null);
            
            for(BufferedImage img : images) {
            	writer.writeToSequence(new IIOImage(img, null, null), params);
            }
            
            writer.endWriteSequence();
        }
        catch(Exception e) {
        	throw new RuntimeException("Failed to combine tiffs.", e);
        }
        finally {
        	IOUtils.closeQuietly(buffer);
        	IOUtils.closeQuietly(output);
        }

		return buffer.toByteArray();
	}

	@Override
	public void writeBytesToFile(File outputFile, byte[] bytes) {
		try {
			FileUtil.write(outputFile, bytes);
		}
		catch(Exception e) {
			throw new RuntimeException("Failed to write bytes to file.", e);
		}
	}
	
// Getters & Setters

// Attributes
	/**
	 * Logger
	 */
	private static final Logger logger = LogManager.getLogger(TiffServiceImpl.class);
	
}
